@@ -7,7 +7,7 @@ require 'sinatra/showexceptions'
 require 'tilt'
 
 module Sinatra
-  VERSION = '1.1.0'
+  VERSION = '1.2.1'
 
   # The request object. See Rack::Request for more info:
   # http://rack.rubyforge.org/doc/classes/Rack/Request.html
@@ -28,6 +28,22 @@ module Sinatra
     else
       alias secure? ssl?
     end
+
+    def forwarded?
+      @env.include? "HTTP_X_FORWARDED_HOST"
+    end
+
+    def route
+      @route ||= begin
+        path = Rack::Utils.unescape(path_info)
+        path.empty? ? "/" : path
+      end
+    end
+
+    def path_info=(value)
+      @route = nil
+      super
+    end
   end
 
   # The response object. See Rack::Response and Rack::ResponseHelpers for
@@ -79,24 +95,37 @@ module Sinatra
 
     # Halt processing and redirect to the URI provided.
     def redirect(uri, *args)
-      if not uri =~ /^https?:\/\//
-        # According to RFC 2616 section 14.30, "the field value consists of a
-        # single absolute URI"
-        abs_uri = "#{request.scheme}://#{request.host}"
-
-        if request.scheme == 'https' && request.port != 443 ||
-              request.scheme == 'http' && request.port != 80
-          abs_uri << ":#{request.port}"
-        end
-
-        uri = (abs_uri << uri)
-      end
-
       status 302
-      response['Location'] = uri
+
+      # According to RFC 2616 section 14.30, "the field value consists of a
+      # single absolute URI"
+      response['Location'] = url(uri, settings.absolute_redirects?, settings.prefixed_redirects?)
       halt(*args)
     end
 
+    # Generates the absolute URI for a given path in the app.
+    # Takes Rack routers and reverse proxies into account.
+    def uri(addr = nil, absolute = true, add_script_name = true)
+      return addr if addr =~ /^https?:\/\//
+      uri = [host = ""]
+      if absolute
+        host << 'http'
+        host << 's' if request.secure?
+        host << "://"
+        if request.forwarded? or request.port != (request.secure? ? 443 : 80)
+          host << request.host_with_port
+        else
+          host << request.host
+        end
+      end
+      uri << request.script_name.to_s if add_script_name
+      uri << (addr ? addr : request.path_info).to_s
+      File.join uri
+    end
+
+    alias url uri
+    alias to uri
+
     # Halt processing and return the error status provided.
     def error(code, body=nil)
       code, body    = 500, code.to_str if code.respond_to? :to_str
@@ -128,10 +157,15 @@ module Sinatra
     # Set the Content-Type of the response body given a media type or file
     # extension.
     def content_type(type, params={})
-      mime_type = mime_type(type)
+      default = params.delete :default
+      mime_type = mime_type(type) || default
       fail "Unknown media type: %p" % type if mime_type.nil?
-      params[:charset] ||= params.delete('charset') || settings.default_encoding
-      response['Content-Type'] = "#{mime_type};#{params.map { |kv| kv.join('=') }.join(', ')}"
+      mime_type = mime_type.dup
+      unless params.include? :charset or settings.add_charset.all? { |p| not p === mime_type }
+        params[:charset] = params.delete('charset') || settings.default_encoding
+      end
+      mime_type << ";#{params.map { |kv| kv.join('=') }.join(', ')}" unless params.empty?
+      response['Content-Type'] = mime_type
     end
 
     # Set the Content-Disposition to "attachment" with the specified filename,
@@ -147,12 +181,11 @@ module Sinatra
     # Use the contents of the file at +path+ as the response body.
     def send_file(path, opts={})
       stat = File.stat(path)
-      last_modified stat.mtime
+      last_modified(opts[:last_modified] || stat.mtime)
 
-      content_type opts[:type] ||
-        File.extname(path) ||
-        response['Content-Type'] ||
-        'application/octet-stream'
+      if opts[:type] or not response['Content-Type']
+        content_type opts[:type] || File.extname(path), :default => 'application/octet-stream'
+      end
 
       if opts[:disposition] == 'attachment' || opts[:filename]
         attachment opts[:filename] || path
@@ -264,7 +297,11 @@ module Sinatra
       end
 
       values = values.map { |value| value.to_s.tr('_','-') }
-      hash.each { |k,v| values << [k.to_s.tr('_', '-'), v].join('=') }
+      hash.each do |key, value|
+        key = key.to_s.tr('_', '-')
+        value = value.to_i if key == "max-age"
+        values << [key, value].join('=')
+      end
 
       response['Cache-Control'] = values.join(', ') if values.any?
     end
@@ -281,12 +318,12 @@ module Sinatra
     def expires(amount, *values)
       values << {} unless values.last.kind_of?(Hash)
 
-      if amount.respond_to?(:to_time)
-        max_age = amount.to_time - Time.now
-        time = amount.to_time
-      else
+      if Integer === amount
+        time    = Time.now + amount
         max_age = amount
-        time = Time.now + amount
+      else
+        time    = time_for amount
+        max_age = time - Time.now
       end
 
       values.last.merge!(:max_age => max_age)
@@ -304,10 +341,10 @@ module Sinatra
     # with a '304 Not Modified' response.
     def last_modified(time)
       return unless time
-      time = time.to_time if time.respond_to?(:to_time)
-      time = Time.parse time.strftime('%FT%T%:z') if time.respond_to?(:strftime)
-      response['Last-Modified'] = time.respond_to?(:httpdate) ? time.httpdate : time.to_s
-      halt 304 if Time.httpdate(request.env['HTTP_IF_MODIFIED_SINCE']) >= time
+      time = time_for time
+      response['Last-Modified'] = time.httpdate
+      # compare based on seconds since epoch
+      halt 304 if Time.httpdate(request.env['HTTP_IF_MODIFIED_SINCE']).to_i >= time.to_i
     rescue ArgumentError
     end
 
@@ -333,9 +370,39 @@ module Sinatra
       end
     end
 
-    ## Sugar for redirect (example:  redirect back)
-    def back ; request.referer ; end
+    # Sugar for redirect (example:  redirect back)
+    def back
+      request.referer
+    end
 
+    private
+
+    # Ruby 1.8 has no #to_time method.
+    # This can be removed and calls to it replaced with to_time,
+    # if 1.8 support is dropped.
+    def time_for(value)
+      if value.respond_to? :to_time
+        value.to_time
+      elsif Time === value
+        value
+      elsif value.respond_to? :new_offset
+        # DateTime#to_time does the same on 1.9
+        d = value.new_offset 0
+        t = Time.utc d.year, d.mon, d.mday, d.hour, d.min, d.sec + d.sec_fraction
+        t.getlocal
+      elsif value.respond_to? :mday
+        # Date#to_time does the same on 1.9
+        Time.local(value.year, value.mon, value.mday)
+      elsif Numeric === value
+        Time.at value
+      else
+        Time.parse value.to_s
+      end
+    rescue ArgumentError => boom
+      raise boom.to_s
+    rescue Exception
+      raise ArgumentError, "unable to convert #{value.inspect} to a Time object"
+    end
   end
 
   # Template rendering methods. Each method takes the name of a template
@@ -347,17 +414,20 @@ module Sinatra
   # that will be rendered.
   #
   # Possible options are:
-  #   :layout       If set to false, no layout is rendered, otherwise
-  #                 the specified layout is used (Ignored for `sass` and `less`)
-  #   :locals       A hash with local variables that should be available
-  #                 in the template
+  #   :content_type   The content type to use, same arguments as content_type.
+  #   :layout         If set to false, no layout is rendered, otherwise
+  #                   the specified layout is used (Ignored for `sass` and `less`)
+  #   :layout_engine  Engine to use for rendering the layout.
+  #   :locals         A hash with local variables that should be available
+  #                   in the template
+  #   :scope          If set, template is evaluate with the binding of the given
+  #                   object rather than the application instance.
+  #   :views          Views directory to use.
   module Templates
     module ContentTyped
       attr_accessor :content_type
     end
 
-    include Tilt::CompileSite
-
     def erb(template, options={}, locals={})
       render :erb, template, options, locals
     end
@@ -386,7 +456,8 @@ module Sinatra
     end
 
     def builder(template=nil, options={}, locals={}, &block)
-      render_xml(:builder, template, options, locals, &block)
+      options[:default_content_type] = :xml
+      render_ruby(:builder, template, options, locals, &block)
     end
 
     def liquid(template, options={}, locals={})
@@ -409,8 +480,8 @@ module Sinatra
       render :radius, template, options, locals
     end
 
-    def markaby(template, options={}, locals={})
-      render :mab, template, options, locals
+    def markaby(template=nil, options={}, locals={}, &block)
+      render_ruby(:mab, template, options, locals, &block)
     end
 
     def coffee(template, options={}, locals={})
@@ -419,14 +490,26 @@ module Sinatra
     end
 
     def nokogiri(template=nil, options={}, locals={}, &block)
-      options[:layout] = false if Tilt::VERSION <= "1.1"
-      render_xml(:nokogiri, template, options, locals, &block)
+      options[:default_content_type] = :xml
+      render_ruby(:nokogiri, template, options, locals, &block)
+    end
+
+    def slim(template, options={}, locals={})
+      render :slim, template, options, locals
+    end
+
+    # Calls the given block for every possible template file in views,
+    # named name.ext, where ext is registered on engine.
+    def find_template(views, name, engine)
+      Tilt.mappings.each do |ext, klass|
+        next unless klass == engine
+        yield ::File.join(views, "#{name}.#{ext}")
+      end
     end
 
   private
     # logic shared between builder and nokogiri
-    def render_xml(engine, template, options={}, locals={}, &block)
-      options[:default_content_type] = :xml
+    def render_ruby(engine, template, options={}, locals={}, &block)
       options, template = template, nil if template.is_a?(Hash)
       template = Proc.new { block } if template.nil?
       render engine, template, options, locals
@@ -435,30 +518,31 @@ module Sinatra
     def render(engine, data, options={}, locals={}, &block)
       # merge app-level options
       options = settings.send(engine).merge(options) if settings.respond_to?(engine)
-      options[:outvar] ||= '@_out_buf'
+      options[:outvar]           ||= '@_out_buf'
+      options[:default_encoding] ||= settings.default_encoding
 
       # extract generic options
       locals          = options.delete(:locals) || locals         || {}
       views           = options.delete(:views)  || settings.views || "./views"
       @default_layout = :layout if @default_layout.nil?
       layout          = options.delete(:layout)
+      eat_errors      = layout.nil?
       layout          = @default_layout if layout.nil? or layout == true
-      content_type    = options.delete(:content_type) || options.delete(:default_content_type)
+      content_type    = options.delete(:content_type)  || options.delete(:default_content_type)
+      layout_engine   = options.delete(:layout_engine) || engine
+      scope           = options.delete(:scope)         || self
 
       # compile and render template
       layout_was      = @default_layout
-      @default_layout = false if layout
+      @default_layout = false
       template        = compile_template(engine, data, options, views)
-      output          = template.render(self, locals, &block)
+      output          = template.render(scope, locals, &block)
       @default_layout = layout_was
 
       # render layout
       if layout
-        begin
-          options = options.merge(:views => views, :layout => false)
-          output = render(engine, layout, options, locals) { output }
-        rescue Errno::ENOENT
-        end
+        options = options.merge(:views => views, :layout => false, :eat_errors => eat_errors, :scope => scope)
+        catch(:layout_missing) { return render(layout_engine, layout, options, locals) { output } }
       end
 
       output.extend(ContentTyped).content_type = content_type if content_type
@@ -466,6 +550,7 @@ module Sinatra
     end
 
     def compile_template(engine, data, options, views)
+      eat_errors = options.delete :eat_errors
       template_cache.fetch engine, data, options do
         template = Tilt[engine]
         raise "Template engine not found: #{engine}" if template.nil?
@@ -477,12 +562,15 @@ module Sinatra
             body = body.call if body.respond_to?(:call)
             template.new(path, line.to_i, options) { body }
           else
-            path = ::File.join(views, "#{data}.#{engine}")
-            Tilt.mappings.each do |ext, klass|
-              break if File.exists?(path)
-              next unless klass == template
-              path = ::File.join(views, "#{data}.#{ext}")
+            found = false
+            find_template(views, data, template) do |file|
+              path ||= file # keep the initial path rather than the last one
+              if found = File.exists?(file)
+                path = file
+                break
+              end
             end
+            throw :layout_missing if eat_errors and not found
             template.new(path, 1, options)
           end
         when data.is_a?(Proc) || data.is_a?(String)
@@ -524,6 +612,7 @@ module Sinatra
       @response = Response.new
       @params   = indifferent_params(@request.params)
       template_cache.clear if settings.reload_templates
+      force_encoding(@request.route)
       force_encoding(@params)
 
       @response['Content-Type'] = nil
@@ -627,11 +716,7 @@ module Sinatra
     # Returns pass block.
     def process_route(pattern, keys, conditions)
       @original_params ||= @params
-      @path ||= begin
-        path = unescape(@request.path_info)
-        path.empty? ? "/" : path
-      end
-      if match = pattern.match(@path)
+      if match = pattern.match(@request.route)
         values = match.captures.to_a
         params =
           if keys.any?
@@ -893,19 +978,25 @@ module Sinatra
         file = (file.nil? || file == true) ? (caller_files.first || File.expand_path($0)) : file
 
         begin
-          app, data =
-            ::IO.read(file).gsub("\r\n", "\n").split(/^__END__$/, 2)
+          io = ::IO.respond_to?(:binread) ? ::IO.binread(file) : ::IO.read(file)
+          app, data = io.gsub("\r\n", "\n").split(/^__END__$/, 2)
         rescue Errno::ENOENT
           app, data = nil
         end
 
         if data
+          if app and app =~ /([^\n]*\n)?#[^\n]*coding: *(\S+)/m
+            encoding = $2
+          else
+            encoding = settings.default_encoding
+          end
           lines = app.count("\n") + 1
           template = nil
+          force_encoding data, encoding
           data.each_line do |line|
             lines += 1
             if line =~ /^@@\s*(.*\S)\s*$/
-              template = ''
+              template = force_encoding('', encoding)
               templates[$1.to_sym] = [template, file, lines]
             elsif template
               template << line
@@ -925,21 +1016,22 @@ module Sinatra
       # Define a before filter; runs before all requests within the same
       # context as route handlers and may access/modify the request and
       # response.
-      def before(path = nil, &block)
-        add_filter(:before, path, &block)
+      def before(path = nil, options = {}, &block)
+        add_filter(:before, path, options, &block)
       end
 
       # Define an after filter; runs after all requests within the same
       # context as route handlers and may access/modify the request and
       # response.
-      def after(path = nil, &block)
-        add_filter(:after, path, &block)
+      def after(path = nil, options = {}, &block)
+        add_filter(:after, path, options, &block)
       end
 
       # add a filter
-      def add_filter(type, path = nil, &block)
+      def add_filter(type, path = nil, options = {}, &block)
         return filters[type] << block unless path
-        block, *arguments = compile!(type, path, block)
+        path, options = //, path if path.respond_to?(:each_pair)
+        block, *arguments = compile!(type, path, block, options)
         add_filter(type) do
           process_route(*arguments) { instance_eval(&block) }
         end
@@ -961,7 +1053,7 @@ module Sinatra
       # Will set params[:agent].
       def user_agent(pattern)
         condition do
-          if request.user_agent =~ pattern
+          if request.user_agent.to_s =~ pattern
             @params[:agent] = $~[1..-1]
             true
           else
@@ -978,7 +1070,7 @@ module Sinatra
         condition do
           matching_types = (request.accept & types)
           unless matching_types.empty?
-            response.headers['Content-Type'] = matching_types.first
+            content_type matching_types.first
             true
           else
             false
@@ -997,18 +1089,18 @@ module Sinatra
         route('HEAD', path, opts, &block)
       end
 
-      def put(path, opts={}, &bk);    route 'PUT',    path, opts, &bk end
-      def post(path, opts={}, &bk);   route 'POST',   path, opts, &bk end
-      def delete(path, opts={}, &bk); route 'DELETE', path, opts, &bk end
-      def head(path, opts={}, &bk);   route 'HEAD',   path, opts, &bk end
+      def put(path, opts={}, &bk)     route 'PUT',     path, opts, &bk end
+      def post(path, opts={}, &bk)    route 'POST',    path, opts, &bk end
+      def delete(path, opts={}, &bk)  route 'DELETE',  path, opts, &bk end
+      def head(path, opts={}, &bk)    route 'HEAD',    path, opts, &bk end
+      def options(path, opts={}, &bk) route 'OPTIONS', path, opts, &bk end
 
     private
       def route(verb, path, options={}, &block)
         # Because of self.options.host
         host_name(options.delete(:host)) if options.key?(:host)
-        options.each { |option, args| send(option, *args) }
 
-        block, pattern, keys, conditions = compile! verb, path, block
+        block, pattern, keys, conditions = compile! verb, path, block, options
         invoke_hook(:route_added, verb, path, block)
 
         (@routes[verb] ||= []).
@@ -1019,7 +1111,8 @@ module Sinatra
         extensions.each { |e| e.send(name, *args) if e.respond_to?(name) }
       end
 
-      def compile!(verb, path, block)
+      def compile!(verb, path, block, options = {})
+        options.each_pair { |option, args| send(option, *args) }
         method_name = "#{verb} #{path}"
 
         define_method(method_name, &block)
@@ -1037,7 +1130,7 @@ module Sinatra
       def compile(path)
         keys = []
         if path.respond_to? :to_str
-          special_chars = %w{. + ( )}
+          special_chars = %w{. + ( ) $}
           pattern =
             path.to_str.gsub(/((:\w+)|[\*#{special_chars.join}])/) do |match|
               case match
@@ -1054,6 +1147,8 @@ module Sinatra
           [/^#{pattern}$/, keys]
         elsif path.respond_to?(:keys) && path.respond_to?(:match)
           [path, path.keys]
+        elsif path.respond_to?(:names) && path.respond_to?(:match)
+          [path, path.names]
         elsif path.respond_to? :match
           [path, keys]
         else
@@ -1097,7 +1192,7 @@ module Sinatra
       end
 
       def quit!(server, handler_name)
-        ## Use thins' hard #stop! if available, otherwise just #stop
+        # Use Thin's hard #stop! if available, otherwise just #stop.
         server.respond_to?(:stop!) ? server.stop! : server.stop
         puts "\n== Sinatra has ended his set (crowd applauds)" unless handler_name =~/cgi/i
       end
@@ -1123,19 +1218,27 @@ module Sinatra
         @prototype ||= new
       end
 
+      # Create a new instance without middleware in front of it.
+      alias new! new unless method_defined? :new!
+
       # Create a new instance of the class fronted by its middleware
       # pipeline. The object is guaranteed to respond to #call but may not be
       # an instance of the class new was called on.
       def new(*args, &bk)
+        build(*args, &bk).to_app
+      end
+
+      # Creates a Rack::Builder instance with all the middleware set up and
+      # an instance of this class as end point.
+      def build(*args, &bk)
         builder = Rack::Builder.new
-        builder.use Rack::Session::Cookie if sessions?
+        setup_sessions builder
         builder.use Rack::CommonLogger    if logging?
         builder.use Rack::MethodOverride  if method_override?
         builder.use ShowExceptions        if show_exceptions?
         middleware.each { |c,a,b| builder.use(c, *a, &b) }
-
-        builder.run super
-        builder.to_app
+        builder.run new!(*args, &bk)
+        builder
       end
 
       def call(env)
@@ -1143,6 +1246,11 @@ module Sinatra
       end
 
     private
+      def setup_sessions(builder)
+        return unless sessions?
+        builder.use Rack::Session::Cookie, :secret => session_secret
+      end
+
       def detect_rack_handler
         servers = Array(server)
         servers.each do |server_name|
@@ -1181,7 +1289,8 @@ module Sinatra
         /\(.*\)/,                                        # generated code
         /rubygems\/custom_require\.rb$/,                 # rubygems require hacks
         /active_support/,                                # active_support require hacks
-        /<internal:/,                                    # internal in ruby >= 1.9.2
+        /bundler(\/runtime)?\.rb/,                       # bundler require hacks
+        /<internal:/                                     # internal in ruby >= 1.9.2
       ]
 
       # add rubinius (and hopefully other VM impls) ignore patterns ...
@@ -1209,20 +1318,22 @@ module Sinatra
     #
     # The latter might not be necessary if Rack handles it one day.
     # Keep an eye on Rack's LH #100.
+    def force_encoding(*args) settings.force_encoding(*args) end
     if defined? Encoding
-      def force_encoding(data)
-        return if data == self || data.is_a?(Tempfile)
+      def self.force_encoding(data, encoding = default_encoding)
+        return if data == settings || data.is_a?(Tempfile)
         if data.respond_to? :force_encoding
-          data.force_encoding settings.default_encoding
+          data.force_encoding encoding
         elsif data.respond_to? :each_value
-          data.each_value { |v| force_encoding(v) }
+          data.each_value { |v| force_encoding(v, encoding) }
         elsif data.respond_to? :each
-          data.each { |v| force_encoding(v) }
+          data.each { |v| force_encoding(v, encoding) }
         end
+        data
       end
     else
-      def force_encoding(*) end
-    end
+      def self.force_encoding(data, *) data end
+      end
 
     reset!
 
@@ -1234,6 +1345,10 @@ module Sinatra
     set :logging, false
     set :method_override, false
     set :default_encoding, "utf-8"
+    set :add_charset, [/^text\//, 'application/javascript', 'application/xml', 'application/xhtml+xml']
+
+    # explicitly generating this eagerly to play nice with preforking
+    set :session_secret, '%x' % rand(2**255)
 
     class << self
       alias_method :methodoverride?, :method_override?
@@ -1246,6 +1361,9 @@ module Sinatra
     set :bind, '0.0.0.0'
     set :port, 4567
 
+    set :absolute_redirects, true
+    set :prefixed_redirects, false
+
     set :app_file, nil
     set :root, Proc.new { app_file && File.expand_path(File.dirname(app_file)) }
     set :views, Proc.new { root && File.join(root, 'views') }
@@ -1329,7 +1447,7 @@ module Sinatra
       end
     end
 
-    delegate :get, :put, :post, :delete, :head, :template, :layout,
+    delegate :get, :put, :post, :delete, :head, :options, :template, :layout,
              :before, :after, :error, :not_found, :configure, :set, :mime_type,
              :enable, :disable, :use, :development?, :test?, :production?,
              :helpers, :settings
