@@ -1,5 +1,6 @@
 # encoding: UTF-8
 require File.dirname(__FILE__) + '/helper'
+require 'erb'
 
 class BaseTest < Test::Unit::TestCase
   setup do
@@ -9,11 +10,11 @@ class BaseTest < Test::Unit::TestCase
 
   it 'allows unicode strings in ascii templates per default (1.9)' do
     next unless defined? Encoding
-    @base.new.haml(File.read(@base.views + "/ascii.haml").encode("ASCII"), {}, :value => "åkej")
+    @base.new.erb(File.read(@base.views + "/ascii.erb").encode("ASCII"), {}, :value => "åkej")
   end
 
   it 'allows ascii strings in unicode templates per default (1.9)' do
     next unless defined? Encoding
-    @base.new.haml(:utf8, {}, :value => "Some Lyrics".encode("ASCII"))
+    @base.new.erb(:utf8, {}, :value => "Some Lyrics".encode("ASCII"))
   end
-end
\ No newline at end of file
+end
