@@ -1,6 +1,8 @@
 ENV['RACK_ENV'] = 'test'
 Encoding.default_external = "UTF-8" if defined? Encoding
 
+RUBY_ENGINE = 'ruby' unless defined? RUBY_ENGINE
+
 begin
   require 'rack'
 rescue LoadError
@@ -23,6 +25,12 @@ class Sinatra::Base
   include Test::Unit::Assertions
 end
 
+class Rack::Builder
+  def include?(middleware)
+    @ins.any? { |m| p m ; middleware === m }
+  end
+end
+
 Sinatra::Base.set :environment, :test
 
 class Test::Unit::TestCase
@@ -53,6 +61,10 @@ class Test::Unit::TestCase
     response.body.to_s
   end
 
+  def assert_body(value)
+    assert_equal value.lstrip.gsub(/\s*\n\s*/, ""), body.lstrip.gsub(/\s*\n\s*/, "")
+  end
+
   # Delegate other missing methods to response.
   def method_missing(name, *args, &block)
     if response && response.respond_to?(name)
