@@ -111,6 +111,18 @@ class HelpersTest < Test::Unit::TestCase
       response = request.get('/', 'SERVER_PORT' => '444')
       assert_equal 'http://example.org:444/foo', response['Location']
     end
+
+    it 'works behind a reverse proxy' do
+      mock_app do
+        get '/' do
+          redirect '/foo'
+        end
+      end
+
+      request = Rack::MockRequest.new(@app)
+      response = request.get('/', 'HTTP_X_FORWARDED_HOST' => 'example.com', 'SERVER_PORT' => '8080')
+      assert_equal 'http://example.com/foo', response['Location']
+    end
   end
 
   describe 'error' do
@@ -243,6 +255,35 @@ class HelpersTest < Test::Unit::TestCase
       follow_redirect!
       assert_equal 'hi bar', body
     end
+
+    it 'inserts session middleware' do
+      mock_app do
+        enable :sessions
+        get '/' do
+          assert env['rack.session']
+          assert env['rack.session.options']
+          'ok'
+        end
+      end
+
+      get '/'
+      assert_body 'ok'
+    end
+
+    it 'sets a default session secret' do
+      mock_app do
+        enable :sessions
+        get '/' do
+          secret = env['rack.session.options'][:secret]
+          assert secret
+          assert_equal secret, settings.session_secret
+          'ok'
+        end
+      end
+
+      get '/'
+      assert_body 'ok'
+    end
   end
 
   describe 'mime_type' do
@@ -320,7 +361,7 @@ class HelpersTest < Test::Unit::TestCase
 
       get '/foo.xml'
       assert ok?
-      assert_equal 'application/foo;charset=utf-8', response['Content-Type']
+      assert_equal 'application/foo', response['Content-Type']
       assert_equal 'I AM FOO', body
     end
 
@@ -334,6 +375,32 @@ class HelpersTest < Test::Unit::TestCase
 
       assert_raise(RuntimeError) { get '/foo.xml' }
     end
+
+    it 'only sets default charset for specific mime types' do
+      tests_ran = false
+      mock_app do
+        mime_type :foo, 'text/foo'
+        mime_type :bar, 'application/bar'
+        mime_type :baz, 'application/baz'
+        add_charset << mime_type(:baz)
+        get '/' do
+          assert_equal content_type(:txt),    'text/plain;charset=utf-8'
+          assert_equal content_type(:css),    'text/css;charset=utf-8'
+          assert_equal content_type(:html),   'text/html;charset=utf-8'
+          assert_equal content_type(:foo),    'text/foo;charset=utf-8'
+          assert_equal content_type(:xml),    'application/xml;charset=utf-8'
+          assert_equal content_type(:xhtml),  'application/xhtml+xml;charset=utf-8'
+          assert_equal content_type(:js),     'application/javascript;charset=utf-8'
+          assert_equal content_type(:bar),    'application/bar'
+          assert_equal content_type(:png),    'image/png'
+          assert_equal content_type(:baz),    'application/baz;charset=utf-8'
+          tests_ran = true
+          "done"
+        end
+      end
+      get '/'
+      assert tests_ran
+    end
   end
 
   describe 'send_file' do
@@ -378,7 +445,7 @@ class HelpersTest < Test::Unit::TestCase
     it 'sets the Content-Type response header if type option is set to a mime type' do
       send_file_app :type => 'application/octet-stream'
       get '/file.txt'
-      assert_equal 'application/octet-stream;charset=utf-8', response['Content-Type']
+      assert_equal 'application/octet-stream', response['Content-Type']
     end
 
     it 'sets the Content-Length response header' do
@@ -393,6 +460,13 @@ class HelpersTest < Test::Unit::TestCase
       assert_equal File.mtime(@file).httpdate, response['Last-Modified']
     end
 
+    it 'allows passing in a differen Last-Modified response header with :last_modified' do
+      time = Time.now
+      send_file_app :last_modified => time
+      get '/file.txt'
+      assert_equal time.httpdate, response['Last-Modified']
+    end
+
     it "returns a 404 when not found" do
       mock_app {
         get '/' do
@@ -415,48 +489,114 @@ class HelpersTest < Test::Unit::TestCase
       assert_equal 'attachment; filename="file.txt"', response['Content-Disposition']
     end
 
+    it "sets the Content-Disposition header when :disposition set to 'inline'" do
+      send_file_app :disposition => 'inline'
+      get '/file.txt'
+      assert_equal 'inline', response['Content-Disposition']
+    end
+
     it "sets the Content-Disposition header when :filename provided" do
       send_file_app :filename => 'foo.txt'
       get '/file.txt'
       assert_equal 'attachment; filename="foo.txt"', response['Content-Disposition']
     end
+
+    it "is able to send files with unkown mime type" do
+      @file = File.dirname(__FILE__) + '/file.foobar'
+      File.open(@file, 'wb') { |io| io.write('Hello World') }
+      send_file_app
+      get '/file.txt'
+      assert_equal 'application/octet-stream', response['Content-Type']
+    end
+
+    it "does not override Content-Type if already set and no explicit type is given" do
+      path = @file
+      mock_app do
+        get '/' do
+          content_type :png
+          send_file path
+        end
+      end
+      get '/'
+      assert_equal 'image/png', response['Content-Type']
+    end
+
+    it "does override Content-Type even if already set, if explicit type is given" do
+      path = @file
+      mock_app do
+        get '/' do
+          content_type :png
+          send_file path, :type => :gif
+        end
+      end
+      get '/'
+      assert_equal 'image/gif', response['Content-Type']
+    end
   end
 
   describe 'cache_control' do
     setup do
-      mock_app {
-        get '/' do
-          cache_control :public, :no_cache, :max_age => 60
+      mock_app do
+        get '/foo' do
+          cache_control :public, :no_cache, :max_age => 60.0
           'Hello World'
         end
-      }
+
+        get '/bar' do
+          cache_control :public, :no_cache
+          'Hello World'
+        end
+      end
     end
 
     it 'sets the Cache-Control header' do
-      get '/'
+      get '/foo'
       assert_equal ['public', 'no-cache', 'max-age=60'], response['Cache-Control'].split(', ')
     end
+
+    it 'last argument does not have to be a hash' do
+      get '/bar'
+      assert_equal ['public', 'no-cache'], response['Cache-Control'].split(', ')
+    end
   end
 
   describe 'expires' do
     setup do
-      mock_app {
-        get '/' do
+      mock_app do
+        get '/foo' do
           expires 60, :public, :no_cache
           'Hello World'
         end
-      }
+
+        get '/bar' do
+          expires Time.now
+        end
+
+        get '/baz' do
+          expires Time.at(0)
+        end
+      end
     end
 
     it 'sets the Cache-Control header' do
-      get '/'
+      get '/foo'
       assert_equal ['public', 'no-cache', 'max-age=60'], response['Cache-Control'].split(', ')
     end
 
     it 'sets the Expires header' do
-      get '/'
+      get '/foo'
       assert_not_nil response['Expires']
     end
+
+    it 'allows passing time objects' do
+      get '/bar'
+      assert_not_nil response['Expires']
+    end
+
+    it 'allows passing time objects' do
+      get '/baz'
+      assert_equal 'Thu, 01 Jan 1970 00:00:00 GMT', response['Expires']
+    end
   end
 
   describe 'last_modified' do
@@ -469,17 +609,18 @@ class HelpersTest < Test::Unit::TestCase
       assert ! response['Last-Modified']
     end
 
-    [Time, DateTime].each do |klass|
-      describe "with #{klass.name}" do
+    [Time.now, DateTime.now, Date.today, Time.now.to_i,
+      Struct.new(:to_time).new(Time.now) ].each do |last_modified_time|
+      describe "with #{last_modified_time.class.name}" do
         setup do
-          last_modified_time = klass.now
           mock_app do
             get '/' do
               last_modified last_modified_time
               'Boo!'
             end
           end
-          @last_modified_time = Time.parse last_modified_time.to_s
+          wrapper = Object.new.extend Sinatra::Helpers
+          @last_modified_time = wrapper.send :time_for, last_modified_time
         end
 
         # fixes strange missing test error when running complete test suite.
@@ -534,6 +675,9 @@ class HelpersTest < Test::Unit::TestCase
             get '/compare', {}, { 'HTTP_IF_MODIFIED_SINCE' => 'Sun, 26 Sep 2010 23:43:52 GMT' }
             assert_equal 200, status
             assert_equal 'foo', body
+            get '/compare', {}, { 'HTTP_IF_MODIFIED_SINCE' => 'Sun, 26 Sep 2100 23:43:52 GMT' }
+            assert_equal 304, status
+            assert_equal '', body
           end
         end
 
@@ -626,6 +770,44 @@ class HelpersTest < Test::Unit::TestCase
     end
   end
 
+  describe 'uri' do
+    it 'generates absolute urls' do
+      mock_app { get('/') { uri }}
+      get '/'
+      assert_equal 'http://example.org/', body
+    end
+
+    it 'includes path_info' do
+      mock_app { get('/:name') { uri }}
+      get '/foo'
+      assert_equal 'http://example.org/foo', body
+    end
+
+    it 'allows passing an alternative to path_info' do
+      mock_app { get('/:name') { uri '/bar' }}
+      get '/foo'
+      assert_equal 'http://example.org/bar', body
+    end
+
+    it 'includes script_name' do
+      mock_app { get('/:name') { uri '/bar' }}
+      get '/foo', {}, { "SCRIPT_NAME" => '/foo' }
+      assert_equal 'http://example.org/foo/bar', body
+    end
+
+    it 'is aliased to #url' do
+      mock_app { get('/') { url }}
+      get '/'
+      assert_equal 'http://example.org/', body
+    end
+
+    it 'is aliased to #to' do
+      mock_app { get('/') { to }}
+      get '/'
+      assert_equal 'http://example.org/', body
+    end
+  end
+
   module ::HelperOne; def one; '1'; end; end
   module ::HelperTwo; def two; '2'; end; end
 
