@@ -40,6 +40,27 @@ class MarkabyTest < Test::Unit::TestCase
     assert_equal "<h1>Markaby Layout!</h1><p>Hello World</p>", body
   end
 
+  it 'renders inline markaby blocks' do
+    markaby_app { markaby { h1 'Hiya' } }
+    assert ok?
+    assert_equal "<h1>Hiya</h1>", body
+  end
+
+  it 'renders inline markaby blocks with inline layouts' do
+    markaby_app do
+      settings.layout { 'h1 { text "THIS. IS. "; yield }' }
+      markaby { em 'SPARTA' }
+    end
+    assert ok?
+    assert_equal "<h1>THIS. IS. <em>SPARTA</em></h1>", body
+  end
+
+  it 'renders inline markaby blocks with file layouts' do    
+    markaby_app { markaby(:layout => :layout2) { text "Hello World" } }
+    assert ok?
+    assert_equal "<h1>Markaby Layout!</h1><p>Hello World</p>", body
+  end
+
   it "raises error if template not found" do
     mock_app { get('/') { markaby :no_such_template } }
     assert_raise(Errno::ENOENT) { get('/') }
@@ -53,6 +74,7 @@ class MarkabyTest < Test::Unit::TestCase
     assert_equal 'foo', body
   end
 end
-rescue
+
+rescue LoadError
   warn "#{$!.to_s}: skipping markaby tests"
 end
