@@ -1,7 +1,7 @@
 require File.dirname(__FILE__) + '/helper'
 
 begin
-require 'rdoc'
+require 'rdoc/markup/to_html'
 
 class RdocTest < Test::Unit::TestCase
   def rdoc_app(&block)
@@ -15,13 +15,13 @@ class RdocTest < Test::Unit::TestCase
   it 'renders inline rdoc strings' do
     rdoc_app { rdoc '= Hiya' }
     assert ok?
-    assert_equal "<h1>Hiya</h1>\n", body
+    assert_body "<h1>Hiya</h1>"
   end
 
   it 'renders .rdoc files in views path' do
     rdoc_app { rdoc :hello }
     assert ok?
-    assert_equal "<h1>Hello From RDoc</h1>\n", body
+    assert_body "<h1>Hello From RDoc</h1>"
   end
 
   it "raises error if template not found" do
@@ -29,6 +29,7 @@ class RdocTest < Test::Unit::TestCase
     assert_raise(Errno::ENOENT) { get('/') }
   end
 end
-rescue
+
+rescue LoadError
   warn "#{$!.to_s}: skipping rdoc tests"
 end
